local SC_WIDTH, SC_HEIGHT = love.graphics.getDimensions()

math.randomseed(os.time())

local Camera = require("camera")


-- Gen stars texture --

local function uniform(a,b)
	return a + (math.random()*(b-a))
end

local function genStars(nStars)

    local canvas = love.graphics.newCanvas()

    love.graphics.setCanvas(canvas)

        local size = uniform(1.7, 3.45)

        for i = 1, nStars do
            love.graphics.rectangle(
                "fill",
                uniform(0, SC_WIDTH-size),
                uniform(0, SC_HEIGHT-size),
                size, size
            )
        end

    love.graphics.setCanvas()

    return canvas

end


-- Shader to repeat the image --

local repeatShader = love.graphics.newShader[[

    // extern number tex_width;     // To be used if the texture is not the same size as the screen, and replace `love_ScreenSize` with these values
    // extern number tex_height;

    extern vec2 cam_pos;

    vec4 effect(vec4 color, Image tex, vec2 tex_coords, vec2 screen_coords)
    {
        number x = mod(screen_coords.x + cam_pos.x, love_ScreenSize.x) / love_ScreenSize.x;
        number y = mod(screen_coords.y + cam_pos.y, love_ScreenSize.y) / love_ScreenSize.y;

        return Texel(tex, vec2(x, y));
    }

]]

local starCanvas = genStars(100)


-- Demo part --

local player = {
    x = (SC_HEIGHT-50)/2,
    y = (SC_WIDTH-50)/2,
    speed = 300
}

local cam = Camera()

function love.update(dt)

    local vx, vy = 0, 0

    if love.keyboard.isScancodeDown("w") then
        vy = vy - 1
    end
    if love.keyboard.isScancodeDown("a") then
        vx = vx - 1
    end
    if love.keyboard.isScancodeDown("s") then
        vy = vy + 1
    end
    if love.keyboard.isScancodeDown("d") then
        vx = vx + 1
    end

    local len = math.sqrt(vx*vx+vy*vy)

    if len > 0 then
        vx = (vx~=0) and (vx / len) * player.speed or 0
        vy = (vy~=0) and (vy / len) * player.speed or 0
    end

    player.x = player.x + vx * dt
    player.y = player.y + vy * dt

    cam:lookAt(player.x, player.y)

    repeatShader:send("cam_pos", {cam.x, cam.y})  -- We send the position of the camera to the shader

end

function love.draw()

    love.graphics.setShader(repeatShader)
        love.graphics.draw(starCanvas)
    love.graphics.setShader()

    cam:attach()
        love.graphics.rectangle("fill", player.x, player.y, 50, 50)
    cam:detach()

end